package server;

import java.util.ArrayList;

/**
 * A simple linked list.
 * @author Martijn
 */
public class SimpleListNode {
	
	/*@
	 * public resource nodeState() = Perm(data, 100)**
	 * Perm(next, 100)**((next != null) -* next.nodeState());
	@*/
	
	/*@
	 * model int size() {
	 * 	int size = 1;
	 * 	SimpleListNode node = this;
	 * 	while (node.next != null) {
	 * 		size++;
	 * 	}
	 * 	return size;
	 * }
	@*/
	
	/*@
	 * model boolean containsLast(String elementData) {
	 * 	SimpleListNode last = this;
	 * 	while (last.next != null) {
	 * 		last = last.next;
	 * 	}
	 * 	return elementData.equals(last.data);
	 * }
	@*/
	
	/*@
	 * model int count(String elementData) {
	 *  int count = 0;
	 * 	SimpleListNode node = this;
	 * 	while (node.next != null) {
	 *  	if(elementData.equals(node.data)) {
	 *  		count ++;
	 *  	}
	 * 		node = node.next;
	 * 	}
	 * 	return count;
	 * }
	@*/
	
	private SimpleListNode next;
	private String data;

	/**
	 * The constructor for this class.
	 * @requires true;
	 * @ensures Perm(next, 100) ** Perm(data, 100);
	 */
	private SimpleListNode() {}
	
	/**
	 * Constructs a list node with the specified data.
	 * @param data The data.
	 * @requires Perm(data, p);
	 * @ensures Perm(next, 100) ** Perm(data, 100) ** next == null;
	 */
	public SimpleListNode(String data) {
		this();
		init(data);
	}
	
	/**
	 * Initializes this node.
	 * @param text The String stored in this node.
	 * @requires Perm(next, 100) ** Perm(data, 100) ** Perm(text, p);
	 * @ensures Perm(next, 100) ** Perm(data, 100) ** next == null;
	 */
	public void init(String text) {
		next = null;
		data = text;
	}

	/**
	 * Adds the specified element to the end of this linked list.
	 * @param elementData The data of the added element.
	 * @requires Perm(elementData, p) ** elementData != null ** nodeState();
	 * @ensures nodeState() ** containsLast(elementData) ** size() == \old(size()) + 1 ** 
	 * 	count(elementData) == \old(count(elementData)+1);
	 */
	public void add(String elementData) {
		if(next == null) {
			next = new SimpleListNode(elementData);
		} else {
			next.add(elementData);
		}
	}

	/**
	 * Removes the first element of this linked list.
	 * @return The removed element or <code>null</code> if the list was empty.
 	 * @requires nodeState();
	 * @ensures nodeState() ** ((\old(next)!=null) -* (next == \old(next.next))) **
	 * 	Perm(removedData, 100) ** count(elementData) == \old(count(elementData))-1;
	 */
	public String remove() {
		String removedData = null;
		if (next != null) {
			removedData = next.data;
			next = next.next;
		}
		return removedData;
	}

	/**
	 * Removes a specific element from this linked list.
	 * @param elementData The data of the element that is removed.
	 * @return The removed element data or <code>null</code> if the element 
	 * 		   was not in the list.
	 * @requires Perm(elementData, p) ** elementData != null ** nodeState();
	 * @ensures nodeState() ** Perm(removedData, 100) ** 
	 * 	Perm(removedData, 100) ** count(elementData) == \old(count(elementData))-1;
	 */
	public String remove(String elementData) {
		String removedData = null;
		if(next != null) {
			if(next.data.equals(elementData)) { // if next node is the one, remove it
				removedData = next.data;
				next = next.next;
			} else { // otherwise, call remove on the next node
				removedData = next.remove(elementData);
			}
		}
		return removedData;
	}
	
	/**
	 * Retrieves the data of this node and all nodes following this node.
	 * @return The retrieved data.
	 * @requires nodeState();
	 * @ensures nodeState() ** Perm(userJobs, 100) ** this.equals(\old(this));
	 */
	public ArrayList<String> retrieveData() {
		ArrayList<String> userJobs = new ArrayList<String>();
		if(next != null) {
			userJobs.add(data);
			userJobs.addAll(next.retrieveData());
			return userJobs;
		} else {
			userJobs.add(data);
			return userJobs;
		}
	}
}
